// An iterable data type to represent the bag data structure.

import java.util.Iterator;

import stdlib.StdIn;
import stdlib.StdOut;

public class Bag<T> implements Iterable<T> {
    private Node first; // front of the bag
    private int n;      // number of items in the bag

    // Constructs an empty bag.
    public Bag() {
        this.first = null;
        this.n = 0;
    }

    // Returns true if this bag is empty, and false otherwise.
    public boolean isEmpty() {
        return this.size() == 0;
    }

    // Returns the number of items in this bag.
    public int size() {
        return this.n;
    }

    // Adds item to this bag.
    public void add(T item) {
        Node oldFirst = this.first;
        this.first = new Node();
        this.first.item = item;
        this.first.next = oldFirst;
        this.n++;
    }

    // Returns a string representation of this bag.
    public String toString() {
        String s = "";
        for (T item : this) {
            s += item + ", ";
        }
        return this.isEmpty() ? s + "[]" : "[" + s.substring(0, s.length() - 2) + "]";
    }

    // Returns an iterator that iterates over the items in this bag.
    public Iterator<T> iterator() {
        return new BagIterator();
    }

    // A data type representing a linked-list of nodes. Each node contains an item and a reference to the next node
    // in the list.
    private class Node {
        private T item;
        private Node next;
    }

    // An iterator that iterates over the items in a bag.
    private class BagIterator implements Iterator<T> {
        private Node current; // current item in the iterator

        // Constructs an iterator.
        public BagIterator() {
            this.current = first;
        }

        // Returns true if there are more items to iterate, and false otherwise.
        public boolean hasNext() {
            return this.current != null;
        }

        // Returns the next item.
        public T next() {
            T item = this.current.item;
            this.current = this.current.next;
            return item;
        }
    }

    // Unit tests the data type.
    public static void main(String[] args) {
        Bag<String> bag = new Bag<String>();
        while (!StdIn.isEmpty()) {
            String item = StdIn.readString();
            if (!item.equals("-")) {
                bag.add(item);
            }
        }
        StdOut.println(bag.size() + " items in the bag");
        StdOut.println(bag);
    }
}
