// A library that implements binary search.

import java.util.Comparator;

import stdlib.In;
import stdlib.StdIn;
import stdlib.StdOut;

public class BinarySearch {
    // Returns the index of key in the sorted array a, or -1.
    public static <T extends Comparable<T>> int indexOf(T[] a, T key) {
        int lo = 0;
        int hi = a.length - 1;
        while (lo <= hi) {
            int mid = lo + (hi - lo) / 2;
            int cmp = key.compareTo(a[mid]);
            if (cmp < 0) {
                hi = mid - 1;
            } else if (cmp > 0) {
                lo = mid + 1;
            } else {
                return mid;
            }
        }
        return -1;
    }

    // Returns the index of key in the sorted array a, or -1 (omparisons are made  according to the order induced by
    // the comparator c).
    public static <T> int indexOf(T[] a, T key, Comparator<T> c) {
        int lo = 0;
        int hi = a.length - 1;
        while (lo <= hi) {
            int mid = lo + (hi - lo) / 2;
            int cmp = c.compare(key, a[mid]);
            if (cmp < 0) {
                hi = mid - 1;
            } else if (cmp > 0) {
                lo = mid + 1;
            } else {
                return mid;
            }
        }
        return -1;
    }

    // Returns the index of key in the sorted array a, or -1.
    public static int indexOf(int[] a, int key) {
        int lo = 0;
        int hi = a.length - 1;
        while (lo <= hi) {
            int mid = lo + (hi - lo) / 2;
            if (key < a[mid]) {
                hi = mid - 1;
            } else if (key > a[mid]) {
                lo = mid + 1;
            } else {
                return mid;
            }
        }
        return -1;
    }

    // Returns the index of key in the sorted array a, or -1.
    public static int indexOf(double[] a, double key) {
        int lo = 0;
        int hi = a.length - 1;
        while (lo <= hi) {
            int mid = lo + (hi - lo) / 2;
            if (key < a[mid]) {
                hi = mid - 1;
            } else if (key > a[mid]) {
                lo = mid + 1;
            } else {
                return mid;
            }
        }
        return -1;
    }

    // Unit tests the library.
    public static void main(String[] args) {
        In in = new In(args[1]);
        if (args[0].equals("String")) {
            String[] whiteList = in.readAllStrings();
            Merge.sort(whiteList);
            while (!StdIn.isEmpty()) {
                String key = StdIn.readString();
                if (indexOf(whiteList, key) == -1) {
                    StdOut.println(key);
                }
            }
        } else if (args[0].equals("string")) {
            String[] whiteList = in.readAllStrings();
            Merge.sort(whiteList, String.CASE_INSENSITIVE_ORDER);
            while (!StdIn.isEmpty()) {
                String key = StdIn.readString();
                if (indexOf(whiteList, key, String.CASE_INSENSITIVE_ORDER) == -1) {
                    StdOut.println(key);
                }
            }
        } else if (args[0].equals("int")) {
            int[] whiteList = in.readAllInts();
            Merge.sort(whiteList);
            while (!StdIn.isEmpty()) {
                int key = StdIn.readInt();
                if (indexOf(whiteList, key) == -1) {
                    StdOut.println(key);
                }
            }
        } else if (args[0].equals("double")) {
            double[] whiteList = in.readAllDoubles();
            Merge.sort(whiteList);
            while (!StdIn.isEmpty()) {
                double key = StdIn.readDouble();
                if (indexOf(whiteList, key) == -1) {
                    StdOut.println(key);
                }
            }
        } else {
            throw new IllegalArgumentException("Illegal command-line argument");
        }
    }
}
