// A data type to represent an individual body in the n-body system.

import stdlib.StdDraw;

public class Body {
    private Vector r;    // current position of the body
    private Vector v;    // current velocity of the body
    private double mass; // mass of the body

    // Constructs a body given its initial position and velocity, and mass.
    public Body(Vector r, Vector v, double mass) {
        this.r = r;
        this.v = v;
        this.mass = mass;
    }

    // Updates the velocity and position of this body based on a force acting on it for a time period.
    public void move(Vector f, double dt) {
        Vector a = f.scale(1 / this.mass);
        this.v = this.v.add(a.scale(dt));
        this.r = this.r.add(this.v.scale(dt));
    }

    // Returns the force on this body due to the other body.
    public Vector forceFrom(Body other) {
        double G = 6.67e-11;
        Vector delta = other.r.subtract(this.r);
        double dist = delta.magnitude();
        double magnitude = (G * this.mass * other.mass) / (dist * dist);
        return delta.direction().scale(magnitude);
    }

    // Draws this body on standard draw.
    public void draw() {
        StdDraw.setPenRadius(0.0125);
        StdDraw.point(this.r.get(0), this.r.get(1));
    }

    // Returns a string representation of this object.
    public String toString() {
        return "(" + this.r + ", " + this.v + ", " + this.mass + ")";
    }

    // Unit tests the data type.
    public static void main(String[] args) {
        StdDraw.setXscale(-5.0e10, 5.0e10);
        StdDraw.setYscale(-5.0e10, 5.0e10);
        double[] aRCoords = {0.0e00, 4.5e10};
        double[] aVCoords = {1.0e04, 0.0e00};
        double[] bRCoords = {0.0e00, -4.5e10};
        double[] bVCoords = {-1.0e04, 0.0e00};
        Body a = new Body(new Vector(aRCoords), new Vector(aVCoords), 1.5e30);
        Body b = new Body(new Vector(bRCoords), new Vector(bVCoords), 1.5e30);
        a.draw();
        b.draw();
        StdDraw.pause(1000);
        Vector fab = a.forceFrom(b);
        Vector fba = b.forceFrom(a);
        a.move(fab, 1000000);
        b.move(fba, 1000000);
        a.draw();
        b.draw();
        StdDraw.show();
    }
}
