// Receives k (int), d (int), and path (String) as command-line inputs; reads a document list as standard input;
// computes d-dimensional profiles based on k-gram frequencies for all those documents under the path directory; and
// writes as standard output a matrix of similarity measures between all pairs of documents.

import stdlib.In;
import stdlib.StdIn;
import stdlib.StdOut;

public class CompareDocuments {
    // Entry point.
    public static void main(String[] args) {
        int k = Integer.parseInt(args[0]);
        int d = Integer.parseInt(args[1]);
        String path = args[2];
        String[] filenames = StdIn.readAllStrings();
        int n = filenames.length;
        Sketch[] sketches = new Sketch[n];
        for (int i = 0; i < n; i++) {
            In inStream = new In(path + "/" + filenames[i]);
            String text = inStream.readAll();
            sketches[i] = new Sketch(text, k, d);
        }
        StdOut.print("    ");
        for (String filename : filenames) {
            StdOut.printf("%8.4s", filename);
        }
        StdOut.println();
        for (int i = 0; i < n; i++) {
            StdOut.printf("%.4s", filenames[i]);
            for (int j = 0; j < n; j++) {
                StdOut.printf("%8.2f", sketches[i].similarTo(sketches[j]));
            }
            StdOut.println();
        }
    }
}
