// An immutable data type to represent a complex number using cartesian coordinates.

import stdlib.StdOut;

public class Complex {
    private double x; // the real part
    private double y; // the imaginary part

    // Constructs a complex number given its cartesian coordinates.
    public Complex(double x, double y) {
        this.x = x;
        this.y = y;
    }

    // Returns the conjugate of this complex number.
    public Complex conjugate() {
        return new Complex(this.x, -this.y);
    }

    // Returns the sum of this and the other complex number.
    public Complex add(Complex other) {
        double x = this.x + other.x;
        double y = this.y + other.y;
        return new Complex(x, y);
    }

    // Returns the difference of this and the other complex number.
    public Complex subtract(Complex other) {
        double x = this.x - other.x;
        double y = this.y - other.y;
        return new Complex(x, y);
    }

    // Returns the product of this and the other complex number.
    public Complex multiply(Complex other) {
        double x = this.x * other.x - this.y * other.y;
        double y = this.x * other.y + this.y * other.x;
        return new Complex(x, y);
    }

    // Returns the magnitude of this complex number.
    public double abs() {
        return Math.sqrt(this.x * this.x + this.y * this.y);
    }

    // Returns true if this and other denote the same complex number, and false otherwise.
    public boolean equals(Object other) {
        if (other == null) {
            return false;
        }
        if (other == this) {
            return true;
        }
        if (other.getClass() != this.getClass()) {
            return false;
        }
        Complex a = this, b = (Complex) other;
        return a.x == b.x && a.y == b.y;
    }

    // Returns a string representation of this complex number.
    public String toString() {
        String SUFFIX = "i";
        if (this.y == 0) {
            return this.x + "";
        } else if (this.x == 0) {
            return this.y + SUFFIX;
        } else if (this.y < 0) {
            return this.x + " - " + (-this.y) + SUFFIX;
        } else {
            return this.x + " + " + this.y + SUFFIX;
        }
    }

    // Unit tests the data type.
    public static void main(String[] args) {
        Complex a = new Complex(5.0, -6.0);
        Complex b = new Complex(3.0, 4.0);
        StdOut.println("a       = " + a);
        StdOut.println("b       = " + b);
        StdOut.println("conj(a) = " + a.conjugate());
        StdOut.println("a + b   = " + a.add(b));
        StdOut.println("a - b   = " + a.subtract(b));
        StdOut.println("a * b   = " + a.multiply(b));
        StdOut.println("|b|     = " + b.abs());
        StdOut.println("a == b  = " + a.equals(b));
    }
}
