// A comparable data type to represent a counter.

import stdlib.StdOut;
import stdlib.StdRandom;

public class Counter implements Comparable<Counter> {
    private String id; // counter name
    private int count; // current value

    // Initializes a new counter with the given id.
    public Counter(String id) {
        this.id = id;
        this.count = 0;
    }

    // Increments this counter by 1.
    public void increment() {
        this.count++;
    }

    // Returns the current value of this counter.
    public int tally() {
        return this.count;
    }

    // Resets this counter to zero.
    public void reset() {
        this.count = 0;
    }

    // Returns true if this and the other counter have the same count, and false otherwise.
    public boolean equals(Object other) {
        if (other == null) {
            return false;
        }
        if (other == this) {
            return true;
        }
        if (other.getClass() != this.getClass()) {
            return false;
        }
        Counter a = this, b = (Counter) other;
        return a.count == b.count;
    }

    // Returns a string representation of this counter.
    public String toString() {
        return this.tally() + " " + this.id;
    }

    // Returns a comparison (-ve, 0, +ve) of this and the other counter by count.
    public int compareTo(Counter other) {
        return Integer.compare(this.tally(), other.tally());
    }

    // Unit tests the data type.
    public static void main(String[] args) {
        int n = Integer.parseInt(args[0]);
        int trials = Integer.parseInt(args[1]);
        Counter[] counters = new Counter[n];
        for (int i = 0; i < n; i++) {
            counters[i] = new Counter("counter " + i);
        }
        for (int t = 0; t < trials; t++) {
            counters[StdRandom.uniform(n)].increment();
        }
        Merge.sort(counters);
        for (Counter counter : counters) {
            StdOut.println(counter);
        }
    }
} 
