// Receives sourceFile (String), targetFile (String), and n (int) as command-line inputs; reads images from
// sourceFile and targetFile; over the course of n frames, gradually replaces the image from sourceFile with the
// image from targetFile; and displays each intermediate image using standard draw.

import stdlib.Picture;
import stdlib.RGBColor;
import stdlib.StdDraw;

public class Fade {
    // Entry point.
    public static void main(String[] args) {
        String sourceFile = args[0];
        String targetFile = args[1];
        int n = Integer.parseInt(args[2]);
        Picture source = new Picture(sourceFile);
        Picture target = new Picture(targetFile);
        int width = source.width();
        int height = source.height();
        Picture picture = new Picture(width, height);
        for (int i = 0; i <= n; i++) {
            for (int col = 0; col < width; col++) {
                for (int row = 0; row < height; row++) {
                    RGBColor c0 = source.get(col, row);
                    RGBColor cn = target.get(col, row);
                    double alpha = 1.0 * i / n;
                    RGBColor c = blend(c0, cn, alpha);
                    picture.set(col, row, c);
                }
            }
            picture.show();
            StdDraw.pause(1000);
        }
    }

    // Returns a new Color object which blends Color objects c1 and c2 using alpha as the blending factor.
    private static RGBColor blend(RGBColor c1, RGBColor c2, double alpha) {
        int r = (int) ((1 - alpha) * c1.getRed() + alpha * c2.getRed());
        int g = (int) ((1 - alpha) * c1.getGreen() + alpha * c2.getGreen());
        int b = (int) ((1 - alpha) * c1.getBlue() + alpha * c2.getBlue());
        return new RGBColor(r, g, b);
    }
}
