// An iterable data type to iterate over the first n numbers from the Fibonacci sequence.

import java.util.Iterator;

import stdlib.StdOut;

public class FibonacciSequence implements Iterable<Long> {
    private int n; // length of the sequence

    // Constructs a FibonacciSequence object given the length of the sequence.
    public FibonacciSequence(int n) {
        this.n = n;
    }

    // Returns a string representation of this object.
    public String toString() {
        String s = "";
        for (long v : this) {
            s += v + " ";
        }
        return s.trim();
    }

    // Returns an iterator that iterates over the numbers in the Fibonacci sequence.
    public Iterator<Long> iterator() {
        return new FibonacciIterator();
    }

    // An iterator that iterates over the numbers in the Fibonacci sequence.
    private class FibonacciIterator implements Iterator<Long> {
        private long a; // previous number in the sequence
        private long b; // current number in the sequence
        private int count; // count of the numbers iterated so far

        // Constructs a FibonacciIterator object.
        public FibonacciIterator() {
            this.a = -1;
            this.b = 1;
            this.count = 0;
        }

        // Returns true if there are more numbers in the sequence, and false otherwise.
        public boolean hasNext() {
            return this.count < n;
        }

        // Returns the next number in the sequence.
        public Long next() {
            this.count++;
            long temp = this.a;
            this.a = this.b;
            this.b += temp;
            return this.b;
        }
    }

    // Unit tests the data type.
    public static void main(String[] args) {
        int n = Integer.parseInt(args[0]);
        FibonacciSequence fib = new FibonacciSequence(n);
        StdOut.println(fib);
    }
}
