// The types defined in this program provide a demonstration of subtyping inheritance in Java.

import stdlib.StdOut;

// Representation of an abstract real-valued function of one variable, f(x).
interface Function {
    // Returns f(x).
    double evaluate(double x);
}

// Representation of the function f(x) = x.
class Line implements Function {
    // Returns f(x) = x.
    public double evaluate(double x) {
        return x;
    }
}

// Representation of the function f(x) = x^2.
class Square implements Function {
    // Returns f(x) = x^2.
    public double evaluate(double x) {
        return x * x;
    }
}

// Test class.
class Functions {
    // Entry point.
    public static void main(String[] args) {
        Function f = new Line();
        Function g = new Square();
        StdOut.println("Integral(x, 0, 1)   = " + integrate(f, 0, 1, 100));
        StdOut.println("Integral(x^2, 0, 1) = " + integrate(g, 0, 1, 100));
    }

    // Returns the area under the curve f(x) between x = a and x = b, computed by dividing the area into n rectangles.
    private static double integrate(Function f, double a, double b, int n) {
        double delta = (b - a) / n;
        double sum = 0.0;
        for (int i = 0; i < n; i++) {
            sum += delta * f.evaluate(a + delta * (i + 0.5));
        }
        return sum;
    }
}
