// A library of Gaussian functions.

import stdlib.StdOut;

public class Gaussian {
    // Returns the value of the Gaussian probability density function with mean 0 and standard deviation 1 at the
    // given z value.
    public static double pdf(double z) {
        return Math.exp(-z * z / 2) / Math.sqrt(2 * Math.PI);
    }

    // Returns the value of the Gaussian probability density function with mean mu and standard deviation sigma at
    // the given x value.
    public static double pdf(double x, double mu, double sigma) {
        double z = (x - mu) / sigma;
        return pdf(z) / sigma;
    }

    // Returns the value of the Gaussian cumulative distribution function with mean 0 and standard deviation 1 at the
    // given z value.
    public static double cdf(double z) {
        if (z < -8.0) {
            return 0.0;
        }
        if (z > 8.0) {
            return 1.0;
        }
        double total = 0.0;
        double term = z;
        int i = 3;
        while (total != total + term) {
            total += term;
            term *= z * z / i;
            i += 2;
        }
        return 0.5 + total * pdf(z);
    }

    // Returns the value of the Gaussian cumulative distribution function with mean mu and standard deviation sigma
    // at the given x value.
    public static double cdf(double x, double mu, double sigma) {
        double z = (x - mu) / sigma;
        return cdf(z);
    }

    // Unit tests the library.
    public static void main(String[] args) {
        double x = Double.parseDouble(args[0]);
        double mu = Double.parseDouble(args[1]);
        double sigma = Double.parseDouble(args[2]);
        StdOut.println(pdf(x, mu, sigma));
        StdOut.println(cdf(x, mu, sigma));
    }
}
