// A data type to represent an undirected symbol graph.

import stdlib.In;
import stdlib.StdOut;

public class Graph {
    private SymbolTable<String, Bag<String>> adj; // maps each vertex to its neighbors
    private int e; // number of edges in graph

    // Constructs an empty graph.
    public Graph() {
        this.adj = new SymbolTable<String, Bag<String>>();
        this.e = 0;
    }

    // Constructs a graph from the given file using the specified delimiter.
    public Graph(String filename, String delimiter) {
        this();
        In inStream = new In(filename);
        while (inStream.hasNextLine()) {
            String line = inStream.readLine();
            String[] names = line.split(delimiter);
            for (int i = 1; i < names.length; i++) {
                this.addEdge(names[0], names[i]);
            }
        }
    }

    // Adds an undirected edge between vertices v and w in this graph.
    public void addEdge(String v, String w) {
        if (!this.adj.contains(v)) {
            this.adj.put(v, new Bag<String>());
        }
        this.adj.get(v).add(w);
        if (!this.adj.contains(w)) {
            this.adj.put(w, new Bag<String>());
        }
        this.adj.get(w).add(v);
        this.e++;
    }

    // Returns the number of vertices in this graph.
    public int countV() {
        return this.adj.size();
    }

    // Returns the number of edges in this graph.
    public int countE() {
        return this.e;
    }

    // Returns the degree of vertex v in this graph.
    public int degree(String v) {
        return this.adj.get(v).size();
    }

    // Returns the vertices adjacent to vertex v in this graph, as an iterable object.
    public Iterable<String> adjacentTo(String v) {
        return this.adj.get(v);
    }

    // Returns all the vertices in this graph, as an iterable object.
    public Iterable<String> vertices() {
        return this.adj.keys();
    }

    // Unit tests the data type.
    public static void main(String[] args) {
        String filename = args[0];
        String delimiter = args[1];
        String v = args[2];
        Graph graph = new Graph(filename, delimiter);
        StdOut.println("V: " + graph.countV());
        StdOut.println("E: " + graph.countE());
        StdOut.print  ("adj(" + v + "): ");
        for (String w : graph.adjacentTo(v)) {
            StdOut.print(w + " ");
        }
        StdOut.println();
    }
}
