// This library implements insertion sort.

import java.util.Comparator;

import stdlib.StdIn;
import stdlib.StdOut;

public class Insertion {
    // Sorts the specified array according to the natural ordering of its objects.
    public static <T extends Comparable<T>> void sort(T[] a) {
        int n = a.length;
        for (int i = 1; i < n; i++) {
            for (int j = i; j > 0 && less(a[j], a[j - 1]); j--) {
                exchange(a, j, j - 1);
            }
        }
    }

    // Sorts the specified array according to the order induced by the specified comparator.
    public static <T> void sort(T[] a, Comparator<T> c) {
        int n = a.length;
        for (int i = 1; i < n; i++) {
            for (int j = i; j > 0 && less(a[j], a[j - 1], c); j--) {
                exchange(a, j, j - 1);
            }
        }
    }

    // Sorts the specified array.
    public static void sort(int[] a) {
        int n = a.length;
        for (int i = 1; i < n; i++) {
            for (int j = i; j > 0 && less(a[j], a[j - 1]); j--) {
                exchange(a, j, j - 1);
            }
        }
    }

    // Sorts the specified array.
    public static void sort(double[] a) {
        int n = a.length;
        for (int i = 1; i < n; i++) {
            for (int j = i; j > 0 && less(a[j], a[j - 1]); j--) {
                exchange(a, j, j - 1);
            }
        }
    }

    // Returns true if v is less than w according to their natural order, and false otherwise.
    private static <T extends Comparable<T>> boolean less(T v, T w) {
        return v.compareTo(w) < 0;
    }

    // Returns true if v is less than w according to the order induced by the comparator c, and false otherwise.
    private static <T> boolean less(T v, T w, Comparator<T> c) {
        return c.compare(v, w) < 0;
    }

    // Returns true if v is less than w, and false otherwise.
    private static boolean less(int v, int w) {
        return v < w;
    }

    // Returns true if v is less than w, and false otherwise.
    private static boolean less(double v, double w) {
        return v < w;
    }

    // Exchanges a[i] and a[j].
    private static void exchange(Object[] a, int i, int j) {
        Object temp = a[i];
        a[i] = a[j];
        a[j] = temp;
    }

    // Exchanges a[i] and a[j].
    private static void exchange(int[] a, int i, int j) {
        int temp = a[i];
        a[i] = a[j];
        a[j] = temp;
    }

    // Exchanges a[i] and a[j].
    private static void exchange(double[] a, int i, int j) {
        double temp = a[i];
        a[i] = a[j];
        a[j] = temp;
    }

    // Unit tests the library.
    public static void main(String[] args) {
        if (args[0].equals("String")) {
            String[] a = StdIn.readAllStrings();
            sort(a);
            for (String s : a) {
                StdOut.print(s + " ");
            }
        } else if (args[0].equals("string")) {
            String[] a = StdIn.readAllStrings();
            sort(a, String.CASE_INSENSITIVE_ORDER);
            for (String s : a) {
                StdOut.print(s + " ");
            }
        } else if (args[0].equals("int")) {
            int[] a = StdIn.readAllInts();
            sort(a);
            for (int i : a) {
                StdOut.print(i + " ");
            }
        } else if (args[0].equals("double")) {
            double[] a = StdIn.readAllDoubles();
            sort(a);
            for (double d : a) {
                StdOut.print(d + " ");
            }
        } else {
            throw new IllegalArgumentException("Illegal command-line argument");
        }
        StdOut.println();
    }
}
