// A library that implements linear search.

import stdlib.In;
import stdlib.StdIn;
import stdlib.StdOut;

import java.util.Comparator;

public class LinearSearch {
    // Returns the index of key in the array a, or -1.
    public static int indexOf(Object[] a, Object key) {
        for (int i = 0; i < a.length; i++) {
            if (a[i].equals(key)) {
                return i;
            }
        }
        return -1;
    }

    // Returns the index of key in the array a, or -1 (comparisons are made using the comparator c).
    public static <T> int indexOf(T[] a, T key, Comparator<T> c) {
        for (int i = 0; i < a.length; i++) {
            if (c.compare(a[i], key) == 0) {
                return i;
            }
        }
        return -1;
    }

    // Returns the index of key in the array a, or -1.
    public static int indexOf(int[] a, int key) {
        for (int i = 0; i < a.length; i++) {
            if (a[i] == key) {
                return i;
            }
        }
        return -1;
    }

    // Returns the index of key in the array a, or -1.
    public static int indexOf(double[] a, double key) {
        for (int i = 0; i < a.length; i++) {
            if (a[i] == key) {
                return i;
            }
        }
        return -1;
    }

    // Unit tests the library.
    public static void main(String[] args) {
        In in = new In(args[1]);
        if (args[0].equals("String")) {
            String[] whiteList = in.readAllStrings();
            while (!StdIn.isEmpty()) {
                String key = StdIn.readString();
                if (indexOf(whiteList, key) == -1) {
                    StdOut.println(key);
                }
            }
        } else if (args[0].equals("string")) {
            String[] whiteList = in.readAllStrings();
            while (!StdIn.isEmpty()) {
                String key = StdIn.readString();
                if (indexOf(whiteList, key, String.CASE_INSENSITIVE_ORDER) == -1) {
                    StdOut.println(key);
                }
            }
        } else if (args[0].equals("int")) {
            int[] whiteList = in.readAllInts();
            while (!StdIn.isEmpty()) {
                int key = StdIn.readInt();
                if (indexOf(whiteList, key) == -1) {
                    StdOut.println(key);
                }
            }
        } else if (args[0].equals("double")) {
            double[] whiteList = in.readAllDoubles();
            while (!StdIn.isEmpty()) {
                double key = StdIn.readDouble();
                if (indexOf(whiteList, key) == -1) {
                    StdOut.println(key);
                }
            }
        } else {
            throw new IllegalArgumentException("Illegal command-line argument");
        }
    }
}
