// Receives xc (double), yc (double), and size (double) as command-line inputs; and draws using standard draw the
// size-by-size region of the Mandelbrot set, centered at (xc, yc).

import stdlib.Picture;
import stdlib.RGBColor;

public class Mandelbrot {
    // Entry point.
    public static void main(String[] args) {
        double xc = Double.parseDouble(args[0]);
        double yc = Double.parseDouble(args[1]);
        double size = Double.parseDouble(args[2]);
        int N = 512;
        int ITERATIONS = 255;
        Picture picture = new Picture(N, N);
        for (int col = 0; col < N; col++) {
            for (int row = 0; row < N; row++) {
                double x0 = xc - size / 2 + size * col / N;
                double y0 = yc - size / 2 + size * row / N;
                Complex z0 = new Complex(x0, y0);
                int gray = ITERATIONS - mandel(z0, ITERATIONS);
                RGBColor color = new RGBColor(gray, gray, gray);
                picture.set(col, N - 1 - row, color);
            }
        }
        picture.show();
    }

    // Returns the number of iterations to check if z0 is in the Mandelbrot set.
    private static int mandel(Complex z0, int iterations) {
        Complex z = z0;
        for (int i = 0; i < iterations; i++) {
            if (z.abs() > 2.0) {
                return i;
            }
            z = z.multiply(z).add(z0);
        }
        return iterations;
    }
}
