// Receives moves (int) as command-line input and a transition matrix as standard input; computes the
// probabilities that a random surfer lands on each page (page ranks) after moves matrix-vector multiplications; and
// writes the page ranks as standard output.

import stdlib.StdArrayIO;
import stdlib.StdOut;

public class Markov {
    // Entry point.
    public static void main(String[] args) {
        int moves = Integer.parseInt(args[0]);
        double[][] transitionMatrix = StdArrayIO.readDouble2D();
        int n = transitionMatrix.length;
        double[] ranks = new double[n];
        ranks[0] = 1.0;
        for (int m = 0; m < moves; m++) {
            double[] newRanks = new double[n];
            for (int j = 0; j < n; j++) {
                for (int i = 0; i < n; i++) {
                    newRanks[j] += ranks[i] * transitionMatrix[i][j];
                }
            }
            ranks = newRanks;
        }
        for (double rank : ranks) {
            StdOut.printf("%8.5f", rank);
        }
        StdOut.println();
    }
}
