// A library of matrix functions.

import stdlib.StdOut;

public class Matrix {
    // Returns the ith row of matrix a.
    public static double[] row(double[][] a, int i) {
        return a[i];
    }

    // Returns the jth column of matrix a.
    public static double[] col(double[][] a, int j) {
        int m = a.length;
        double[] c = new double[m];
        for (int i = 0; i < m; i++) {
            c[i] = a[i][j];
        }
        return c;
    }

    // Returns the sum of the matrices a and b.
    public static double[][] add(double[][] a, double[][] b) {
        int m = a.length, n = a[0].length;
        double[][] c = new double[m][n];
        for (int i = 0; i < m; i++) {
            for (int j = 0; j < n; j++) {
                c[i][j] = a[i][j] + b[i][j];
            }
        }
        return c;
    }

    // Returns the difference of matrices a and b.
    public static double[][] subtract(double[][] a, double[][] b) {
        int m = a.length, n = a[0].length;
        double[][] c = new double[m][n];
        for (int i = 0; i < m; i++) {
            for (int j = 0; j < n; j++) {
                c[i][j] = a[i][j] - b[i][j];
            }
        }
        return c;
    }

    // Returns the product of matrices a and b.
    public static double[][] multiply(double[][] a, double[][] b) {
        int m = a.length, n = b[0].length;
        double[][] c = new double[m][n];
        for (int i = 0; i < m; i++) {
            for (int j = 0; j < n; j++) {
                c[i][j] = dot(row(a, i), col(b, j));
            }
        }
        return c;
    }

    // Returns the transpose of matrix a.
    public static double[][] transpose(double[][] a) {
        int m = a.length, n = a[0].length;
        double[][] c = new double[n][m];
        for (int i = 0; i < m; i++) {
            for (int j = 0; j < n; j++) {
                c[j][i] = a[i][j];
            }
        }
        return c;
    }

    // Returns the dot product of the 1-by-n matrices a and b.
    public static double dot(double[] a, double[] b) {
        double total = 0.0;
        int n = a.length;
        for (int i = 0; i < n; i++) {
            total += a[i] * b[i];
        }
        return total;
    }

    // Returns a string representation of 1-by-n matrix a.
    public static String toString(double[] a) {
        String s = "[";
        for (int i = 0; i < a.length; i++) {
            s += a[i] + ", ";
        }
        return a.length == 0 ? s + "]" : s.substring(0, s.length() - 2) + "]";
    }

    // Returns a string representation of matrix a.
    public static String toString(double[][] a) {
        String s = "[";
        for (int i = 0; i < a.length; i++) {
            s += toString(a[i]) + ", ";
        }
        return a.length == 0 ? s + "]" : s.substring(0, s.length() - 2) + "]";
    }

    // Unit tests the library.
    public static void main(String[] args) {
        double[][] a = {{1, 2, 3}, {4, 5, 6}, {7, 8, 9}};
        double[][] b = {{1}, {2}, {3}};
        StdOut.println("a              = " + toString(a));
        StdOut.println("b              = " + toString(b));
        StdOut.println("row(a, 1)      = " + toString(row(a, 1)));
        StdOut.println("col(a, 1)      = " + toString(col(a, 1)));
        StdOut.println("add(a, a)      = " + toString(add(a, a)));
        StdOut.println("subtract(a, a) = " + toString(subtract(a, a)));
        StdOut.println("multiply(a, b) = " + toString(multiply(a, b)));
        StdOut.println("transpose(b)   = " + toString(transpose(b)));
    }
}
