// A data type to represent shortest paths within an undirected symbol graph from a fixed source vertex.

import stdlib.StdOut;

public class PathFinder {
    private Graph graph; // the graph
    private String s; // the source vertex
    private SymbolTable<String, Integer> distTo; // maps a vertex to its distance from source
    private SymbolTable<String, String> edgeTo; // maps a vertex to previous vertex on path

    // Constructs a path finder given the graph and source vertex.
    public PathFinder(Graph graph, String s) {
        this.graph = graph;
        this.s = s;
        this.distTo = new SymbolTable<String, Integer>();
        this.edgeTo = new SymbolTable<String, String>();
        Queue<String> queue = new Queue<String>();
        queue.enqueue(s);
        this.distTo.put(s, 0);
        while (!queue.isEmpty()) {
            String v = queue.dequeue();
            for (String w : graph.adjacentTo(v)) {
                if (!this.distTo.contains(w)) {
                    queue.enqueue(w);
                    this.distTo.put(w, 1 + this.distTo.get(v));
                    this.edgeTo.put(w, v);
                }
            }
        }
    }

    // Returns the distance of vertex v from the source vertex.
    public int distanceTo(String v) {
        return this.distTo.get(v);
    }

    // Returns true if there is a path to vertex v from the source vertex, and false otherwise.
    public boolean hasPathTo(String v) {
        return this.distTo.contains(v);
    }

    // Returns the path to vertex v from the source vertex.
    public Iterable<String> pathTo(String v) {
        Stack<String> path = new Stack<String>();
        for (String x = v; x != null && this.distTo.contains(x); x = this.edgeTo.get(x)) {
            path.push(x);
        }
        return path;
    }

    // Returns a string representation of this object.
    public String toString() {
        StringBuilder sb = new StringBuilder();
        for (String t : graph.vertices()) {
            if (hasPathTo(t)) {
                sb.append(this.s + " -> " + t + ": ");
                for (String v : pathTo(t)) {
                    sb.append(v + " ");
                }
                sb.append("(" + distanceTo(t) + ")\n");
            }
        }
        return sb.toString().strip();
    }

    // Unit tests the data type.
    public static void main(String[] args) {
        String filename = args[0];
        String delimiter = args[1];
        String s = args[2];
        Graph graph = new Graph(filename, delimiter);
        PathFinder pf = new PathFinder(graph, s);
        StdOut.println(pf);
    }
}
