// A library of percolation functions.

import stdlib.StdArrayIO;
import stdlib.StdOut;

public class Percolation {
    // Computes and returns the full sites of the given percolation system.
    public static boolean[][] flow(boolean[][] isOpen) {
        int n = isOpen.length;
        boolean[][] isFull = new boolean[n][n];
        for (int j = 0; j < n; j++) {
            flow(isOpen, isFull, 0, j);
        }
        return isFull;
    }

    // Given the full sites of a percolation system, returns true if the system percolates, and false otherwise.
    public static boolean percolates(boolean[][] isFull) {
        int n = isFull.length;
        for (int j = 0; j < n; j++) {
            if (isFull[n - 1][j]) {
                return true;
            }
        }
        return false;
    }

    // Given the open and full sites of a percolation system, updates the full sites by marking
    // every site of that system that is open and reachable from site (i, j).
    private static void flow(boolean[][] isOpen, boolean[][] isFull, int i, int j) {
        int n = isOpen.length;
        if (i < 0 || i >= n || j < 0 || j >= n) {
            return;
        }
        if (!isOpen[i][j] || isFull[i][j]) {
            return;
        }
        isFull[i][j] = true;
        flow(isOpen, isFull, i + 1, j);
        flow(isOpen, isFull, i, j + 1);
        flow(isOpen, isFull, i, j - 1);
        flow(isOpen, isFull, i - 1, j);
    }

    // Unit tests the library.
    public static void main(String[] args) {
        boolean[][] isOpen = StdArrayIO.readBoolean2D();
        boolean[][] isFull = flow(isOpen);
        StdArrayIO.print(isFull);
        StdOut.println(percolates(isFull));
    }
}
