// Receives n (int) as command-line input; and draws using standard draw a plot of the percolation probability
// (experimental observation) against the vacancy probability (control variable).

import stdlib.StdDraw;

public class PercolationPlot {
    // Entry point.
    public static void main(String[] args) {
        int n = Integer.parseInt(args[0]);
        StdDraw.setCanvasSize(750, 750);
        StdDraw.setXscale(-0.2, 1.2);
        StdDraw.setYscale(-0.2, 1.2);
        StdDraw.setPenRadius(0.0);
        StdDraw.square(0.5, 0.5, 0.52);
        StdDraw.setPenColor(StdDraw.WHITE);
        StdDraw.filledSquare(0.5, 0.5, 0.51);
        StdDraw.setPenColor(StdDraw.BLACK);
        StdDraw.text(0.5, -0.1, "Vacancy Probability");
        StdDraw.text(-0.11, 0.5, "Percolation");
        StdDraw.text(-0.11, 0.45, "Probability");
        curve(n, 0.0, 0.0, 1.0, 1.0, 10000, 0.01, 0.0025);
    }

    // Plots the percolation curve (percolation probability vs vacancy probability) recursively.
    private static void curve(int n, double x0, double y0, double x1, double y1, int trials, double gap, double error) {
        double xm = (x0 + x1) / 2;
        double ym = (y0 + y1) / 2;
        double fxm = evaluate(n, xm, trials);
        if (x1 - x0 < gap || Math.abs(ym - fxm) < error) {
            StdDraw.line(x0, y0, x1, y1);
            return;
        }
        curve(n, x0, y0, xm, fxm, trials, gap, error);
        StdDraw.filledCircle(xm, fxm, 0.005);
        if (xm > 0.55 && xm < 0.6) {
            String xyLabel = String.format("(%.3f, %.3f)", xm, fxm);
            StdDraw.text(xm + 0.1, fxm, xyLabel);
        }
        curve(n, xm, fxm, x1, y1, trials, gap, error);
    }

    // Generates a random n-by-n percolation system with vacancy probability p and determines if the system
    // percolates. Repeats trials times. Returns an estimate of the empirical percolation probability of such systems.
    private static double evaluate(int n, double p, int trials) {
        int count = 0;
        for (int i = 0; i < trials; i++) {
            boolean[][] isOpen = PercolationIO.random(n, p);
            boolean[][] isFull = Percolation.flow(isOpen);
            if (Percolation.percolates(isFull)) {
                count++;
            }
        }
        return 1.0 * count / trials;
    }
}
