// Receives sound samples, each characterized by a pitch and a duration, as standard input; and plays the sound using
// standard audio.

import stdlib.StdAudio;
import stdlib.StdIn;

public class PlayThatTuneDeluxe {
    // Entry point.
    public static void main(String[] args) {
        while (!StdIn.isEmpty()) {
            int pitch = StdIn.readInt();
            double duration = StdIn.readDouble();
            StdAudio.play(createRichNote(pitch, duration));
        }
    }

    // Returns an array of samples for a note superposed from three notes (at pitch, 2 x pitch,
    // and 0.5 x pitch) and having the specified duration.
    private static double[] createRichNote(double pitch, double duration) {
        int NOTES_ON_SCALE = 12;
        double CONCERT_A = 440.0;
        double hz = CONCERT_A * Math.pow(2, pitch / NOTES_ON_SCALE);
        double[] mid = createNote(hz, duration);
        double[] hi = createNote(2 * hz, duration);
        double[] lo = createNote(hz / 2, duration);
        double[] hiAndLo = superpose(hi, lo, 0.5, 0.5);
        return superpose(mid, hiAndLo, 0.5, 0.5);
    }

    // Returns an array of samples for a note of specified frequency and duration.
    private static double[] createNote(double hz, double duration) {
        int n = (int) (StdAudio.SAMPLE_RATE * duration);
        double[] note = new double[n + 1];
        for (int i = 0; i <= n; i++) {
            note[i] = Math.sin(2 * Math.PI * i * hz / StdAudio.SAMPLE_RATE);
        }
        return note;
    }

    // Superposes arrays a and b, using respective weights aWeight and bWeight, and returns the
    // superposed array.
    private static double[] superpose(double[] a, double[] b, double aWeight, double bWeight) {
        double[] c = new double[a.length];
        for (int i = 0; i < a.length; i++) {
            c[i] = a[i] * aWeight + b[i] * bWeight;
        }
        return c;
    }
}
