// The types defined in this program provide a demonstration of subclassing inheritance in Java.

import stdlib.StdOut;

// Representation of an abstract shape.
class Shape {
    // Returns the area of this shape.
    public double area() {
        return Double.NaN;
    }
}

// Representation of a rectangle.
class Rectangle extends Shape {
    private double width;  // width of the rectangle
    private double height; // height of the rectangle

    // Constructs a rectangle given its width and height.
    public Rectangle(double width, double height) {
        this.width = width;
        this.height = height;
    }

    // Returns the area of this rectangle.
    public double area() {
        return this.width * this.height;
    }

    // Returns a string representation of this rectangle.
    public String toString() {
        return "Rectangle(w = " + this.width + ", h = " + this.height + ")";
    }
}

// Representation of a circle.
class Circle extends Shape {
    private double radius; // radius of the circle

    // Constructs a circle given its radius.
    public Circle(double radius) {
        this.radius = radius;
    }

    // Returns the area of this rectangle.
    public double area() {
        return Math.PI * this.radius * this.radius;
    }

    // Returns a string representation of this circle.
    public String toString() {
        return "Circle(r = " + this.radius + ")";
    }
}

// Test class.
public class Shapes {
    // Entry point.
    public static void main(String[] args) {
        Rectangle r = new Rectangle(3, 4);
        Circle c = new Circle(5);
        StdOut.println("r        = " + r);
        StdOut.println("r.area() = " + r.area());
        StdOut.println("c        = " + c);
        StdOut.println("c.area() = " + c.area());
    }
}
