// An immutable data type that represents a profile of a string, as a d-dimensional unit vector.

import stdlib.StdOut;

public class Sketch {
    private Vector sketch; // string profile as a d-dimensional unit vector

    // Constructs a new sketch which is a profile of text, as a d-dimensional unit vector. Component i of the vector
    // indicates how many k-grams in the text hash to i.
    public Sketch(String text, int k, int d) {
        double[] freq = new double[d];
        for (int i = 0; i < text.length() - k + 1; i++) {
            String kgram = text.substring(i, i + k);
            int hash = kgram.hashCode();
            freq[Math.abs(hash % d)]++;
        }
        Vector vector = new Vector(freq);
        this.sketch = vector.direction();
    }

    // Returns the similarity measure between this and the other sketch, as a number between 0 and 1. The value 0
    // indicates that the sketches are dissimilar, and 1 indicates that they are similar.
    public double similarTo(Sketch other) {
        return this.sketch.dot(other.sketch);
    }

    // Returns a string representation of this sketch.
    public String toString() {
        return this.sketch.toString();
    }

    // Unit tests the data type.
    public static void main(String[] args) {
        String a = args[0];
        Sketch sketchA = new Sketch(a, 2, 3);
        String b = args[1];
        Sketch sketchB = new Sketch(b, 2, 3);
        StdOut.println("a                          = " + a);
        StdOut.println("sketchA                    = " + sketchA);
        StdOut.println("b                          = " + b);
        StdOut.println("sketchB                    = " + sketchB);
        StdOut.println("sketchA.similarTo(sketchB) = " + sketchA.similarTo(sketchB));
    }
}
