// A data type for turtle graphics using standard draw.

import stdlib.StdDraw;

public class Turtle {
    private double x; // x-coordinate of turtle
    private double y; // y-coordinate of turtle
    private double theta; // ccw angle (in degrees) of turtle

    // Constructs a turtle given its coordinates and angle.
    public Turtle(double x, double y, double theta) {
        this.x = x;
        this.y = y;
        this.theta = theta;
    }

    // Rotates this turtle by theta in ccw direction.
    public void turnLeft(double theta) {
        this.theta += theta;
    }

    // Moves this turtle forward by given amount, with the pen down.
    public void goForward(double stepSize) {
        double xOld = this.x;
        double yOld = this.y;
        this.x += stepSize * Math.cos(Math.toRadians(this.theta));
        this.y += stepSize * Math.sin(Math.toRadians(this.theta));
        StdDraw.line(xOld, yOld, this.x, this.y);
    }

    // Returns a string representation of this turtle.
    public String toString() {
        return "(" + this.x + ", " + this.y + ", " + this.theta + ")";
    }

    // Unit tests the data type.
    public static void main(String[] args) {
        int n = Integer.parseInt(args[0]);
        Turtle turtle = new Turtle(0.5, 0.0, 180.0 / n);
        double stepSize = Math.sin(Math.toRadians(180.0 / n));
        StdDraw.setPenRadius(0.001);
        for (int i = 0; i < n; i++) {
            turtle.goForward(stepSize);
            turtle.turnLeft(360.0 / n);
        }
        StdDraw.show();
    }
}
