// A data type to represent a universe.

import stdlib.StdDraw;
import stdlib.In;

public class Universe {
    private Body[] bodies; // array of n bodies

    // Constructs an n-body universe from the given file containing the number (n) of bodies, their initial positions
    // and velocities, and their masses.
    public Universe(String filename) {
        In inStream = new In(filename);
        int n = inStream.readInt();
        double radius = inStream.readDouble();
        StdDraw.setXscale(-radius, +radius);
        StdDraw.setYscale(-radius, +radius);
        this.bodies = new Body[n];
        for (int i = 0; i < n; i++) {
            double rx = inStream.readDouble();
            double ry = inStream.readDouble();
            double vx = inStream.readDouble();
            double vy = inStream.readDouble();
            double mass = inStream.readDouble();
            double[] position = {rx, ry};
            double[] velocity = {vx, vy};
            Vector r = new Vector(position);
            Vector v = new Vector(velocity);
            this.bodies[i] = new Body(r, v, mass);
        }
    }

    // Updates the state of this universe to what it would be after the given time period.
    public void increaseTime(double dt) {
        int n = this.bodies.length;
        Vector[] f = new Vector[n];
        for (int i = 0; i < n; i++) {
            f[i] = new Vector(new double[2]);
        }
        for (int i = 0; i < n; i++) {
            for (int j = 0; j < n; j++) {
                if (i != j) {
                    f[i] = f[i].add(this.bodies[i].forceFrom(this.bodies[j]));
                }
            }
        }
        for (int i = 0; i < n; i++) {
            this.bodies[i].move(f[i], dt);
        }
    }

    // Draws this universe to standard draw.
    public void draw() {
        for (Body body : this.bodies) {
            body.draw();
        }
    }

    // Returns a string representation of this object.
    public String toString() {
        String s = "";
        for (Body body : this.bodies) {
            s += body + ", ";
        }
        s = this.bodies.length > 0 ? "[" + s.substring(0, s.length() - 2) + "]" : "[]";
        return s;
    }

    // Unit tests the data type.
    public static void main(String[] args) {
        String filename = args[0];
        Universe universe = new Universe(filename);
        universe.draw();
        StdDraw.show();
        StdDraw.pause(1000);
        universe.increaseTime(1000000);
        universe.draw();
        StdDraw.show();
    }
}
