// An iterable data type to iterate over the words in a sentence.

import java.util.Iterator;

import stdlib.StdOut;

public class Words implements Iterable<String> {
    private String[] words; // words in the sentence

    // Constructs a Words object from the given sentence.
    public Words(String sentence) {
        this.words = sentence.split("\\s+");
    }

    // Returns a string representation of this object.
    public String toString() {
        String s = "";
        for (String v : this) {
            s += v + " ";
        }
        return s.trim();
    }

    // Returns an iterator to iterate over the words in a sentence.
    public Iterator<String> iterator() {
        return new WordsIterator();
    }

    // An iterator that iterates over the words in a sentence.
    private class WordsIterator implements Iterator<String> {
        private int current; // index of the current word

        // Constructs a WordsIterator object.
        public WordsIterator() {
            this.current = 0;
        }

        // Returns true if there are more words in the sentence, and false otherwise.
        public boolean hasNext() {
            return this.current < words.length;
        }

        // Returns the next word in the sentence.
        public String next() {
            return words[this.current++];
        }
    }

    // Unit tests the data type.
    public static void main(String[] args) {
        String sentence = args[0];
        Words words = new Words(sentence);
        StdOut.println(words);
    }
}
